from persistent import Persistent
from repoze.folder import Folder
from time import time

from pybookbuilder.utils import get_default_file, extract_title

class RootContainer(Folder):
    __parent__ = __name__ = None

def appmaker(zodb_root):
    if not 'app_root' in zodb_root:
        app_root = RootContainer()
        zodb_root['app_root'] = app_root
        book_folder = BookFolder()
        app_root['books'] = book_folder
        import transaction
        transaction.commit()
    return zodb_root['app_root']

# ------------------------------------------------

class BookFolder(Folder):
    """ Base container for Book objects, used for view purposes.
    Just a simple subclass of repoze.folder.Folder.
    """

    pass

class Book(Folder):
    """ Book object, representing a book in pyBookBuilder.
    """
    
    def __init__(self, title):
        super(Book, self).__init__()
        self.title = title

        self.index_rst = get_default_file('index.rst').read()
        self.css = get_default_file('style.css').read()
        self.conf_py = get_default_file('conf.py').read()

        # Replace the fill-in title with an underlined title
        ul_title = title+'\n'+'='*len(title)
        self.index_rst = self.index_rst % {'title':ul_title}
        
        self['chapters'] = ChapterFolder()
        self['assets'] = AssetFolder()

class ChapterFolder(Folder):
    """ Container for Chapter objects, used for view purposes.
    Just a simple subclass of repoze.folder.Folder.
    """

    pass

class Chapter(Persistent):        
    """ Chapter object, representing a chapter in pyBookBuilder.
    """

    def __init__(self, title, number):
        super(Chapter, self).__init__()
        self.number = number
        underlined_title = title + '\n' + '='*len(title)
        self.data = underlined_title + '\n'

        self.mtime = 0
        self.title_gentime = -1
        self.regen_title()

    def regen_title(self):
        self._title = extract_title(self.data)

    @property
    def title(self):
        if self.title_gentime < self.mtime:
            self.regen_title()
        return self._title

class AssetFolder(Folder):
    """ Container for Asset objects, used for view purposes.
    Just a simple subclass of repoze.folder.Folder.
    """

    pass

class Asset(Persistent):        
    """ Asset object, representing an user-uploaded asset (image, etc) in
    pyBookBuilder.
    """

    pass
