from deform import Form, ValidationFailure
from pyramid.httpexceptions import HTTPFound
from pyramid.renderers import get_renderer
from pyramid.url import resource_url
from pyramid.view import view_config

from pybookbuilder.resources import BookFolder, Book
from pybookbuilder.utils import choose_name
from pybookbuilder.views.forms import BookSchema

@view_config(context=BookFolder, renderer='pybookbuilder:templates/book_list.pt')
def book_list(context, request):
    renderer = get_renderer('pybookbuilder:templates/layout.pt')
    new_book_url = resource_url(context, request, 'create_book')

    books = []
    for book in context.values():
        tup = (book, resource_url(book,request))
        books.append(tup)
        
    return {'macro': renderer.template,
            'new_book_url': new_book_url,
            'books': books}


@view_config(context=BookFolder, name="create_book",
             renderer="pybookbuilder:templates/book_create.pt")
def create_book(context, request):
    renderer = get_renderer('pybookbuilder:templates/layout.pt')
    form = Form(BookSchema(), buttons=('submit',))

    if 'submit' in request.POST:
        controls = request.POST.items()

        try:
            data = form.validate(controls)
        except ValidationFailure, e:
            return {'macro': renderer.template,
                    'form': e.render()}

        title = data['title']
        book = Book(title)
        name = choose_name(book, context)
        context[name] = book
        redirect_url = resource_url(book, request)
        return HTTPFound(headers={'location':redirect_url})

    return {'macro': renderer.template,
            'form': form.render()}

@view_config(context=Book, renderer="pybookbuilder:templates/book.pt")
def view_book(context, request):
    renderer = get_renderer('pybookbuilder:templates/layout.pt')
    chapters = list(context['chapters'].values())
    chapters.sort(key=lambda x: x.number)
    chapters = [(c, resource_url(c,request)) for c in chapters]
    chapter_create_url = resource_url(context['chapters'], request, 'create_chapter')
    book_list_url = resource_url(context.__parent__, request)
    return {'macro': renderer.template,
            'chapters': chapters,
            'chapter_create_url': chapter_create_url,
            'book_list_url': book_list_url} 
