from deform import Form, ValidationFailure
from pyramid.httpexceptions import HTTPFound
from pyramid.renderers import get_renderer
from pyramid.response import Response
from pyramid.url import resource_url
from pyramid.view import view_config

from pybookbuilder.resources import ChapterFolder, Chapter
from pybookbuilder.utils import choose_name
from pybookbuilder.views.forms import ChapterAddSchema

@view_config(context=ChapterFolder)
def chapterfolder_redirect(context, request):
    redirect_url = resource_url(context.__parent__, request)
    return HTTPFound(headers={'location': redirect_url})

@view_config(context=ChapterFolder, name="create_chapter",
             renderer="pybookbuilder:templates/chapter_create.pt")
def create_chapter(context, request):
    renderer = get_renderer('pybookbuilder:templates/layout.pt')
    form = Form(ChapterAddSchema(), buttons=('submit',))
    book = context.__parent__
    
    if 'submit' in request.POST:
        controls = request.POST.items()
        try:
            data = form.validate(controls)
        except ValidationFailure, e:
            return {'macro':renderer.template,
                    'form':e.render(),
                    'book':book}
        title = data['title']
        number = data['number']
        chapter = Chapter(title, number)
        name = choose_name(chapter, context)
        context[name] = chapter
        redirect_url = resource_url(chapter, request)
        return HTTPFound(headers={'location':redirect_url})

    return {'macro': renderer.template,
            'form': form.render(),
            'book': book}

@view_config(context=Chapter, renderer="pybookbuilder:templates/chapter.pt")
def view_chapter(context, request):
    renderer = get_renderer('pybookbuilder:templates/layout.pt')
    book = context.__parent__.__parent__
    book_url = resource_url(book, request)
    ajax_save_url = resource_url(context, request, 'ajax_save')

    return {'macro': renderer.template,
            'book': book,
            'book_url': book_url,
            'ajax_save_url': ajax_save_url}
    
@view_config(context=Chapter, name="ajax_save")
def save_chapter_ajax(context, request):
    context.data = request.body
    return Response('OK')
